#include <glib.h>
#include <glib-object.h>

#ifndef GEGO_GLOBAL_NOTIFY_HEADER
#define GEGO_GLOBAL_NOTIFY_HEADER

G_BEGIN_DECLS

#define GEGO_TYPE_GLOBEL_NOTIFY (gego_global_notify_get_type ())
G_DECLARE_FINAL_TYPE (GegoGlobalNotify, gego_global_notify, GEGO, GLOBAL_NOTIFY, GObject)


typedef struct _GegoGlobalNotifyPrivate GegoGlobalNotifyPrivate;

typedef struct _GegoCbackItem GegoCallbackItem;

struct
_GegoGlobalNotify {
  GObject parent_instance;
};

GType
gego_global_notify_get_type ();

static void
gego_global_notify_init (GegoGlobalNotify * );


gboolean
gego_global_notify_initalize (GError ** err);


gboolean
gego_global_notify_uninitalize (GError ** err);

gint
gego_global_notify_add_notification (gchar * name, GError ** err);


void
gego_global_notify_remove_notification (gchar * name,
                                        gint notification_id,
                                        GError ** err);

gint
gego_global_notify_subscribe (gchar * name, GCallback callback,
                              gpointer subscriber_data,
                              GError ** err);

void
gego_global_notify_unsubscribe (gchar * name,
                                gint subscription_id,
                                GError ** err);

gint
gego_global_notify_call (gchar * name,
                         gpointer caller_data,
                         GError ** err);

gboolean
gego_global_notify_is_initialized (void);

/**
 * gego_global_notify_return_not_initialized:
 * @short_description: convinience macro; returns with an error if Global Notify
 * is not initialized.
 *
 * @err: (type GError) An #GError to be passed to the macro.
 */
#define gego_global_notify_return_not_initialized(err)\
  if (!gego_global_notify_is_initialized ()) {\
    g_set_error (err, GEGO_GLOBAL_NOTIFY_ERROR,\
                       GEGO_GLOBAL_NOTIFY_ERROR_NOT_INITIALZED,\
                       "Global Notify is not initialized.");\
    g_return_if_reached();\
  }


GQuark
gego_global_notify_error_quark (void);

/**
 * GEGO_GLOBAL_NOTIFY_ERROR:
 * @short_description: A convinience wrapper around #gego_global_notify_error_quark()
 */
#define GEGO_GLOBAL_NOTIFY_ERROR gego_global_notify_error_quark ()

typedef enum {
  GEGO_GLOBAL_NOTIFY_ERROR_NONE,
  GEGO_GLOBAL_NOTIFY_ERROR_ALREADY_INITALIZED,
  GEGO_GLOBAL_NOTIFY_ERROR_NOT_INITIALZED,
  GEGO_GLOBAL_NOTIFY_ERROR_SIGNAL_ALREADY_EXISTS,
  GEGO_GLOBAL_NOTIFY_ERROR_SIGNAL_DOES_NOT_EXIST,
  GEGO_GLOBAL_NOTIFY_ERROR_INVALID_SUBSCRIPTION_ID,
  GEGO_GLOBAL_NOTIFY_ERROR_LAST
} GegoGlobalNotifyError;

/**
 * GegoGlobalNotifyErrorName:
 * @short_description: Get the name of the error.
 */
gchar *
GegoGlobalNotifyErrorName[] __attribute__((unused)) = {
  "NONE",
  "Already initialized",
  "Not initialized",
  "Signal already exists",
  "Signal does not exist",
  "Invalid subscription id",
  0x0
};


gchar *
gego_global_notify_error_get_name(GegoGlobalNotifyError k);

G_END_DECLS

#endif /* GEGO_GLOBAL_NOTIFY_HEADER */
